function udpxpctargetsteam
% UDPXPCTARGETSTREAM  Demo of capablity to stream data from xPC Target to 
%   the host computer.  This demo has two Simulink models: 'udphost', which
%   runs on the Host computer with Simulink and 'udptarget', which executes
%   on an xPC Target computer.  This pair of models creates a simple 
%   protocol on top of the standard UDP blocks to create a fast and reliable 
%   data stream from the xPC Target to the Host computer.  This arrangement 
%   can be used to access a a continuous stream of signal values on the host 
%   computer 
%
% Parameters:
% Packetsize - This value defines the 
%
% Packetbuffer - this value defines how many complete packets
%
%  Memory (bytes) = packetbuffer * packetsize * sizeof(data) * numel(dat);
%
%   Where 
%   sizeof(data) - is the number of bytes in the type of data.  for
%      example, a double precision value requires 8 bytes.
%   numel(data) - Array size of the data.  For example, if the data is a
%     vector of size [1 3], the number of elements is 3.  While a [2 3]
%     matrix has 6 elements.  
%

% Copyright 2004-2005 The MathWorks, Inc.

% Compute (near) optimum packet size:
datasize = 12;  % sizeof(double) + sizeof(uint32)
packetsize = floor(1492/datasize);
packetsizebytes = packetsize*datasize;

samptime = 0.00005;   % xPC Target base Sample time (100 usec)
packetbuffer = 200;   % Number of packets to buffer 
datasocket = 25000
acksocket = datasocket+1;
hostip = gethostip;
targetip = gettargetip;
% Place these values in the base workspace (necessary for models to locate
% values)
assignin('caller','datasocket',datasocket);
assignin('caller','acksocket',acksocket);
assignin('caller','datasize',datasize);
assignin('caller','samptime',samptime);
assignin('caller','packetsize',packetsize);
assignin('caller','packetsizebytes',packetsizebytes);
assignin('caller','packetbuffer',packetbuffer);
assignin('caller','hostip',hostip);
assignin('caller','targetip',targetip);

% Compute and display worst case host latency, should 
disp(['The host computer must have a worst case latency less than = ',...
    num2str(samptime*packetbuffer),' sec']);

% Load models and set values (IP addresses, which are not evaluated)
hostsys = load_system('udphost');
targetsys = load_system('udptarget');

% Write the IP addresses of the host and target into the appropriate
% block parameters (UDP blocks)
hostudpblks = find_system('udphost','regexp','on','ipAddress', '.*');
for iblks = 1:numel(hostudpblks),
    set_param(hostudpblks{iblks},'ipAddress',targetip);
end

targetudpblks = find_system('udptarget','regexp','on','ipAddress', '.*');
for iblks = 1:numel(targetudpblks),
    set_param(targetudpblks{iblks},'ipAddress',hostip);
end

% Build and download your target model.  
open_system('udptarget');
%return;
make_rtw

open_system('udphost');
set_param(bdroot,'SimulationCommand', 'start');

tg = xpc;
start(tg)

disp('Manually stop the updhost model, when the xPC Target model has completed it''s execution');
disp(' The integer value on the target scope shows the number of dropped values (Hopefully zero!');
disp(' If data values were dropped, the generated data file will contain gaps in the data   ');
disp('To check results, load and execute the model udphostpost, which unpacks the raw data');
%% GETTARGETIP ------------------------------------------------------------
function ip = gettargetip
% Determine Target computers IP address by check values configured by xPC
% Target Manager (default target)
% Note - this can be replaced by a static IP address, if necessary:
%  ip = 100.100.100.100;
ip = getxpcenv('TcpIpTargetAddress');
%ip = ip{1};

%% GETHOSTIP --------------------------------------------------------------
function ip = gethostip
% Get Host computers IP address by querying the windows command-line utility:
% ipconfig.
% Note - this can be replaced by a static IP address, if necessary:
%  ip = 100.100.100.100;
[s,w] = dos('ipconfig');
if isempty(w) 
   error([' Unable to determine IP address of host computer. ',char(10),... 
         ' To continue, replace the hostip funtion call with host IP address']);
end
expr = '(?<=IP Address\D+)\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}';
[ebeg,eend] = regexp(w,expr,'start','end');
if isempty(ebeg) || isempty(eend),
    error([' Unable to determine IP address of host computer. ',char(10),... 
         ' To continue, replace the hostip funtion call with host IP address']);
end
ip = w(ebeg:eend);
